from ._place_holder import Figure_
from .Graph import Graph
import matplotlib.pyplot as plt
# import warnings


def addFigure(self: Figure_,
              template: str = "default",
              rows: int = 1,
              cols: int = 1
              ) -> None:
    """Create a new figure.

    Parameters
    ----------

    *   ``template``: Set the template used by the figure, i.e. all the font
        sizes used by the different components.
    *   ``rows``: Set the horizontal size of the figure by multiplying the
        default size of a plot (found in the template used) by this value.
        ``rows`` should thus be equal to the maximum amount of plots that will
        be placed horizontally.
    *   ``cols``: Set the vertical size of the figure by multiplying the
        default size of a plot (found in the template used) by this value.
        ``cols`` should thus be equal to the maximum amount of plots that will
        be placed vertically.
    """

    if(template in self.templates):
        self.template = self.templates[template]
    else:
        raise NameError("The template'", template, "'does not exist.")

    x_size = self.template["fig_size_x"]*cols
    y_size = self.template["fig_size_y"]*rows
    self.fig = plt.figure(figsize=(x_size, y_size))

    # warnings.warn("New figure created")


def addGraph(self: Figure_,
             name: str,
             row: int = 1,
             col: int = 1,
             index: int = 1) -> Graph:
    """Add a graph to the current figure.

    Parameters:
    --------

    *   ``name``: name of the graph, used to recognize this specific graph
    *   ``row``: row number
    *   ``col``: column number
    *   ``index``: location of the graph

    Details:
    --------
    Explanation: specify the location and size of the graph on the figure.
    The row and col parameters determine the size of the grid, like if the
    figure was a table. Then, the index parameter specify in which block of
    the table the added graph has to go starting from the upper left corner
    and ending at the lower right. \n
    /!\\ the table system doesn't actually exist, it is only there to give a
    position and a size to the added graph. It is thus possible to use
    different size for different graphs. For example when using a 2x2 grid,
    plotting two graphs with (2,2,1) and (2,2,2) with a third one (2,1,2) will
    gives the following structure: \n
    _______\n
    |__|__|\n
    |_____|
    """
    new_graph = Graph(self, self.fig.add_subplot(row, col, index))
    self.graphs[name] = new_graph

    # warnings.warn(f'Graph "{name}" created')
    return self.graphs[name]


def setTitle(self: Figure_, title: str) -> None:
    """Set the main title of the figure.

    Parameters
    ----------

    *   ``title``: well, obvious

    """
    self.fig.suptitle(title+"\n", fontsize=self.template["fig_title_size"])


def figSave(self: Figure_, name: str) -> None:
    """Save the figure to pdf format.

    Parameters
    ----------

    *   ``name``: figure saved as "name.pdf"

    """
    self.fig.tight_layout()
    self.fig.savefig(name+".pdf", bbox_inches='tight')
    # warnings.warn(f'Figure saved as "{name+".pdf"}"')


def figShow(self: Figure_):
    """Don't remember

    """
    self.fig.tight_layout()
    plt.show()



import matplotlib.pyplot as plt
from matplotlib.collections import QuadMesh


class Graph__():
    def __init__(self):
        self.plot_labels = [[], []]

        self._x_axis_params: dict[str, bool] = dict()

        self._y_axis_params: dict[str, bool] = dict()

        self._is_legend_plotted: bool = False
        self._mappable: QuadMesh
        
        self.secondYAxis: plt.Axes



from .._place_holder import Graph_
from typing import Literal, Union

import datetime
import matplotlib.ticker as tick
import matplotlib.pyplot as plt


def setAxisX(self: Graph_, x_min: Union[int, float, None] = None,
             x_max: Union[int, float, None] = None,
             label: Union[str, None] = None, color: list = ['black', False],
             loc: Literal['center', 'right', 'left']= 'center'
             ):
    """Set the labels and the interval of the X axis of the current graph.

    Parameters
    ----------

    *   ``x_min``: (OPTIONNAL) the lower bound of the axis.
    *   ``x_max``: (OPTIONNAL) upper bound of the axis.
    *   ``label``: (OPTIONNAL) name of the axis (don't forget the units !).
    *   ``color``: (OPTIONNAL) color of the label.
    *   ``loc``: (OPTIONNAL) location of the label ('left', 'center', 'right').

    """
    if label is not None:
        self.plot.set_xlabel(label, color=color[0], loc=loc,
                             fontsize=self.fig.template["label_size"])
    self.plot.tick_params(axis='x', which='major',
                          labelsize=self.fig.template["tick_size"],
                          width=self.fig.template["tick_width_major"],
                          length=self.fig.template["tick_length_major"]
                          )
    self.plot.tick_params(axis='x', which='minor',
                          labelsize=self.fig.template["tick_size"],
                          width=self.fig.template["tick_width_minor"],
                          length=self.fig.template["tick_length_minor"]
                          )
    if(color[1]):
        self.plot.tick_params(axis='x', which='major', colors=color[0])
    if(x_min is not None and x_max is not None):
        self.plot.set_xlim(x_min, x_max)


def setAxisXAngularScale(self: Graph_, span: float = 1.0):
    """Change the x axis to a multiple of pi axis.

    Parameters
    ----------

    *   ``span``: (DEF=1) set the span between two ticks.
    """
    self._x_axis_params["angular"] = True
    self.plot.xaxis.set_major_formatter(tick.FormatStrFormatter('%g$\\pi$'))
    self.plot.xaxis.set_major_locator(tick.MultipleLocator(base=span))


def setAxisXLogScale(self: Graph_):
    """Change the x axis to a base 10 logarithm scale.
    """
    self._x_axis_params["log"] = True


def setAxisXTimeScale(self: Graph_):
    """ Set the X axis as a time axis in hh:mm:ss.
    """

    self._x_axis_params["time"] = True

    def timeTicks(xx, pos):
        d = datetime.timedelta(seconds=xx)
        return str(d)

    self.plot.yaxis.get_offset_text().set_fontsize(25)
    self.plot.ticklabel_format(scilimits=(0, 0))
    formatter = tick.FuncFormatter(timeTicks)
    self.plot.xaxis.set_major_formatter(formatter)

    plt.setp(self.plot.xaxis.get_majorticklabels(),
             rotation=-30,
             ha="left", rotation_mode="anchor"
             )


def setAxisY(self: Graph_,
             y_min: Union[int, float, None] = None,
             y_max: Union[int, float, None] = None,
             label: Union[str, None] = None,
             color: list = ['black', False],
             loc: Literal['bottom', 'center', 'top'] = 'center'
             ):
    """Set the labels and the interval of the Y axis of the current graph.

    Parameters
    ----------

    *   ``x_min``: (OPTIONNAL) the lower bound of the axis.
    *   ``x_max``: (OPTIONNAL) upper bound of the axis.
    *   ``label``: (OPTIONNAL) name of the axis (don't forget the units !).
    *   ``color``: (OPTIONNAL) color of the label.
    *   ``loc``: (OPTIONNAL) location of the label ('top', 'center', 'bottom').

    """
    if label is not None:
        self.plot.set_ylabel(label,
                             color=color[0],
                             loc=loc,
                             fontsize=self.fig.template["label_size"]
                             )
    if(color[1]):
        self.plot.tick_params(axis='y', which='major', colors=color[0])
    self.plot.tick_params(axis='y', which='major',
                          labelsize=self.fig.template["tick_size"],
                          width=self.fig.template["tick_width_major"],
                          length=self.fig.template["tick_length_major"]
                          )
    self.plot.tick_params(axis='y', which='minor',
                          labelsize=self.fig.template["tick_size"],
                          width=self.fig.template["tick_width_minor"],
                          length=self.fig.template["tick_length_minor"]
                          )
    if(y_min is not None and y_max is not None):
        self.plot.set_ylim(y_min, y_max)


def setAxisYAngularScale(self: Graph_, span: float = 1.0):
    """Change the y axis to a multiple of pi axis.

    Parameters
    ----------

    *   ``span``: (DEF = 1) set the span between two ticks.
    """
    self._y_axis_params["angular"] = True
    self.plot.yaxis.set_major_formatter(tick.FormatStrFormatter('%g$\\pi$'))
    self.plot.yaxis.set_major_locator(tick.MultipleLocator(base=span))


def setAxisYLogScale(self: Graph_):
    """Change the y axis to a base 10 logarithm scale.
    """
    self._y_axis_params["log"] = True


def setAxisYSecondAxis(self: Graph_,
                       y_min: Union[int, float, None] = None,
                       y_max: Union[int, float, None] = None,
                       label: Union[str, None] = None,
                       color: list = ['black', False],
                       loc: Literal['bottom', 'center', 'top'] = 'center'
                       ):
    """Create and set the labels and the interval of the second Y axis of the
    current graph.

    Parameters
    ----------

    *   ``x_min``: (OPTIONNAL) the lower bound of the axis.
    *   ``x_max``: (OPTIONNAL) upper bound of the axis.
    *   ``label``: (OPTIONNAL) name of the axis (don't forget the units !).
    *   ``color``: (OPTIONNAL) color of the label.
    *   ``loc``: (OPTIONNAL) location of the label ('top', 'center', 'bottom').

    """
    self._y_axis_params["second"] = True
    self.secondYAxis = self.plot.twinx()
    if label is not None:
        self.secondYAxis.set_ylabel(label, color=color[0], loc=loc,
                                    fontsize=self.fig.template["label_size"]
                                    )
    if(color[1]):
        self.secondYAxis.tick_params(axis='y', which='major',
                                     colors=color[0])
    self.secondYAxis.tick_params(axis='y', which='major',
                                 labelsize=self.fig.template["tick_size"]
                                 )
    if(y_min is not None and y_max is not None):
        self.secondYAxis.set_ylim(y_min, y_max)
